SDF #-*- shell-script -*-

# Stereo Pipeline Configuration File
# Created : Eric Zbinden 09/96 zbinden@artemis.arc.nasa.gov
# Updated : Eric Zbinden 06/98
# Intelligent Mechanisms Group 
# NASA Ames Research Center
# Moffett Field, CA 94035

##################
##     MODE	##
##################

# verbose mode off=0, bombastic=5
VERBOSE 0

# mosaic the images onto a spherical billboad (No=0/Yes=1)
MOSAIC 0

# set which camera/eye will be used as a reference (left=0 or right=1)
# # # # # # # # # KURT=0 ERIC=1
REFERENCE_CAMERA 0
# # # # # # # # # KURT=ERIC=0
MASTER_EYE 0

##################################
##	CAMERA PARAMETERS	##
##################################

# camera parameters units are [mm] and [mrad]

BASELINE	  150
TILT_PIVOT_OFFSET 12
CAMERA_OFFSET 62.3
H_THETA_L_PIXEL    0.9917
V_THETA_L_PIXEL    0.9917
H_THETA_R_PIXEL    0.9973
V_THETA_R_PIXEL    0.9973
L_TOE_IN_0	 -13.945
R_TOE_IN_0	  16.45 14.282
BORESIGHT_0	   3.87
IMG_ROTATION	   3.3

USE_MOTOR_COUNT 1

#############################
##	PREPROCESSING	   ##
#############################

DO_FORMAT_IMG_SIZE 0
DO_SLOG 0
DO_FIRST_HIST_EQ 1
DO_EMBOSS 1
DO_SECOND_HIST_EQ 0
AUTO_SET_CORR_PARAM 1
AUTO_SET_V_CORR_PARAM 0
DO_VERT_CAL 0
APPLY_MASK 0
WRITE_PREPROCESSED 0
WRITE_TEXTURE 0
WRITE_MASK 0

RED_CHANEL_FACTOR   1.0
GREEN_CHANEL_FACTOR 1.0
BLUE_CHANEL_FACTOR  1.0

#	  mpl	    mpl EM
ALIGN.a   1.0	    0.994384
ALIGN.b   0.0	    -0.0033	
ALIGN.c	  0.0	    0.0033
ALIGN.d   1.0	    0.994384
ALIGN.Tx  0.0	    0.2900
# # # # # # # # # KURT=0.78 ERIC=3.88
ALIGN.Ty  0.78 2.78	    3.8800

H_TIE_PTS 21
V_TIE_PTS 21

ALIGN_MARGIN_%_REJECT 1

# cropping of the stereo image
# only done if DP_FORMAT_IMG_SIZE is on
CROP_X_MIN 00
CROP_X_MAX 10000
CROP_Y_MIN 00
CROP_Y_MAX 10000

# max output image size if OUT_WIDTH or OUT_HEIGHT = 0 
#  => automatic calculation to conserve w/h ratio
OUT_WIDTH 320
OUT_HEIGHT 0

# mask low contrast treshold value
MASK_LOW_CONTRAST_TRESHOLD 0.8

# center surround diameter of the convolution kernel in the SLOG 
SLOG_KERNEL_WIDTH 3.0

#########################################################################
###########################    CORRELATION    ###########################
#########################################################################

DO_CORRELATION 1
2D_CORRELATION 1
CORR_1_CLEAN_UP 1
DO_2nd_PASS 1
CORR_2_CLEAN_UP 1
DO_3rd_PASS 0
CORR_3_CLEAN_UP 0
WRITE_DEBUG_DISP 0
WRITE_DISP_STP 0
WRITE_DISP_PGM 0
# # # # # # # # # # Kurt=1 Eric=0 
WRITE_DISP_VICAR 1

#sub pixel resolution with quadratic approximation validity limit in pixel
SUB_PXL_TRESHOLD 0.5

# 1st pass correlation kernel size (bound disparity range)
#  (11/9)(7/3)(9/5)
H_KERNEL_1 21
V_KERNEL_1 15

# second pass (subpixel estimation)
CORR_MARGIN_2 3
H_KERNEL_2 13
V_KERNEL_2 11

# third pass (parallax corrected kernel)
CORR_MARGIN_3 5
H_KERNEL_3 11
V_KERNEL_3 11

# correlation window size
H_CORR_MIN -50
H_CORR_MAX  150
V_CORR_MIN  -4
V_CORR_MAX  -1

#########################################################################
############################    FILTERING    ############################
#########################################################################

FILL_V_HOLES 0
FILL_H_HOLES 1
EXTEND_DISP_LR 0
EXTEND_DISP_TB 0
SMOOTH_DISP 0 
# # # # # # # # # # # # # # Kurt=1 Eric=0
WRITE_EXTRAPOLATION_MASK 1
WRITE_FILTERED_DISP_PGM 0
SMOOTH_RANGE 0

# automatic removal of the low confidence pixels
#  default values 5 5 60 10 
#  note 5*5 halfkernel = 11*11 kernel -> 121 pixels 
RM_H_HALF_KERN 3  5
RM_V_HALF_KERN 3  5
RM_MIN_MATCHES 20 60
RM_TRESHOLD 200   150

# fill hole functions 
H_FILL_TRESHOLD 4
V_FILL_TRESHOLD 2 

# extend the disparity map of the Left and right
EXTEND_DISP_L 30
EXTEND_DISP_R 30
EXTEND_DISP_T 7
EXTEND_DISP_B 7

# offset to add at the extrapolated rows: (in DISP_PER_PXL)
# positif numbers offset (Top & Bottom) as expected on a flat terrain 
OFFSET_DISP_T 50
OFFSET_DISP_B 50

# size of the matrix to smooth the disparity map
SM_DISP_M 9 
SM_DISP_N 9 

# smooth range or disp 
# if 0 then skip the function
SMR_TRESHOLD 300

# correction for the distortion
X_DISP_CORRECTION 0.0
Y_DISP_CORRECTION 0.0
DISP_CORR_OFFSET 0.0

# second order correction to compensate the lens aberation
# disp -= (A2*r^2 + A1*r + A0)*DISP_PER_PXL (r=0 @ center and 1 @ the edge)
A2 0.0
A1 0.0
A0 0.0

#########################################################################
#############################    DOTCLOUD    ############################
#########################################################################

# # # # # # # # # # Kurt=0 Eric=1
DO_DOTCLOUD 0
FAR_FIELD_BILLBOARD 0
DO_LOCAL_LEVEL_TRANSFORM 0
WRITE_DOTCLOUD 0
WRITE_MVACS_RANGE 0
WRITE_VICAR_XYZ 0

# (-143.25 plus right toe in -17.19 mrad -> -0.98491445 mrad)
PAN_OFFSET -144.2349144 
TILT_OFFSET 0.0

# size max of the universe in meters and altitude of the ground
UNIVERSE_RADIUS 30.0
GROUND_PLANE_LEVEL -0.87

# model scale (the following offset are scaled in the code
#              and are not modified when changing scale)
MODEL_SCALE 1.0

# world's origin offset [m] ( all zero for pathfinder )
X_OFFSET 0.0	+0.5300
Y_OFFSET 0.0	-0.2324
Z_OFFSET 0.7937 -0.0063 

# Mars Pathfinder Local Level transform parameters [rad] & [m]
# angle btwn the x axis and the zero motor click axis
# stowed ??? deployed 41.75-138.25-143.25
IMP_AZ_OFFSET 2.500184

# offset btwn az/el axis and imp origin
# stowed -0.1757 deployed -0.7937
IMP_CAN_Z_OFFSET 0.7937

# IMP frame and Lander frame [m]
X_IMP_OFFSET  0.0 -0.5300
Y_IMP_OFFSET  0.0 -0.2324
Z_IMP_OFFSET  0.0 -0.8000

# quaternion values [qw, (qx, qy, qz)]  (lander to local level)

# updated quarternion
#LOCAL_LEVEL_W   0.2102
#LOCAL_LEVEL_X  -0.0146
#LOCAL_LEVEL_Y  -0.0293
#LOCAL_LEVEL_Z   0.9771 

#########################################################################
#############################    TEXTURE     ############################
#########################################################################

TEXTURE_CASTING_TYPE 1
MIN_GRAY_IN_TEXTURE  0
MAX_GRAY_IN_TEXTURE  4095

#########################################################################
#########################    ALTITUDE TEXTURE    ########################
#########################################################################

DO_ALTITUDE_TEXTURE 1

# altitude texture parameter in [m]
# mode 1 periodic 0 saturation outside [offset ; offset+range]
ALTITUDE_MODE 0
ALTITUDE_RANGE 0.6
ALTITUDE_OFFSET -1.0

# hsv color for altitude 
# CC=> top>bottom CW=>bottom<top (add 360 if necessary)
# 0.red 60.yellow 120.green 180.cyan 240.blue 300.magenta 360.red ...
ALT_TOP_COLOR 0
ALT_BOTTOM_COLOR 240

# constrast factor for img texture overlay  0.none 1.max 
TEXTURE_CONTRAST 0.5

#########################################################################
#############################    3D MESH    #############################
#########################################################################

DO_3D_MESH 1
ADAPTIVE_MESHING 0
DOUBLE_SIDED 0
NFF_PLAIN 0
NFF_TXT 0
INVENTOR 0
VRML 1

# terrain construction
MOSAIC_V_STEP 82
MOSAIC_H_STEP 85

NFF_V_STEP 5
NFF_H_STEP 5
NFF_MAX_JUMP 500
NFF_2D_MAP 0

OUT_MESH_SCALE 1.0

MESH_TOLERANCE 0.002
MAX_TRIANGLES 10000

# material and shapehints properties

AMBIENT_RED	0.8
AMBIENT_GREEN	0.8
AMBIENT_BLUE	0.8

DIFFUSE_RED	0.2
DIFFUSE_GREEN	0.2
DIFFUSE_BLUE	0.2

SPECULAR_RED	0.0
SPECULAR_GREEN	0.0
SPECULAR_BLUE	0.0

EMISSIVE_RED	0.5
EMISSIVE_GREEN	0.5
EMISSIVE_BLUE	0.5

SHININESS	0.2
TRANSPARENCY	0.0
CREASE_ANGLE	1.5
SHAPE_TYPE_SOLID 1

WRITE_TEXTURE_SWITCH 0

#########################################################################
#############################  EXPLANATION  #############################
#########################################################################

# MOSAIC		When the mosaic option is on (1) the terrain
#			model is not computed but the images are
#			projected onto a sphere of radius UNIVERSE_RADIUS.
#			off = 0, on = 1
#
# VERBOSE		No need to explain: off=0, bombastic=5
#


# BASELINE		distance between the Left and Right Cameras
#			The baseline of the camera is expected to be 
#			horizontal [mm]
#
# TILT_PIVOT_OFFSET	vertical distance between the tilt axis of the
#			pan/tilt and the z axis of the camera whith
#			the z axis horizontal [mm]
#
# CAMERA_OFFSET		horizontal distance between the tilt axis of the
#			pan/tilt and the focal point of the camera whith
#			the z axis horizontal [mm]
#
# H_THETA_*_PIXEL	Horizontal Scale or Horizontal field of view
#			per pixel for the L/R camera [mrad]
#
# V_THETA_*_PIXEL	Vertical Scale or Vertical field of view
#			per pixel for the L/R camera [mrad]
#
# *_TOE_IN_0		toe in (angle between the z axis and the
#			normal of the baseline in the baseline/z axis
#			plane) Counted positive CounterClockwise when
#			seen from the top [mrad]
#
# BORESIGHT_0		Angle made by the projection of the Left and 
#			Right z axis onto a vertical plane (go to check
#			which way is positive and if this is
#			implemented) [mrad]
#


# DO_FORMAT_IMG_SIZE	if turned on (1) it will subsample the imput 
#			raw image and output a OUT_WIDTH*OUT_HEIGHT 
#			pixelssize image. If either OUT_WIDTH or
#			OUT_HEIGHT is equal to zero its value will 
#			be automatically computed so the aspect ratio 
#			of the image is conserved. (on = 1; off = 0).
#
# DO_FIRST_HIST_EQ	This function, if turned on (1), will force
#			the left and right image histogram to be
#			more similar. The adjustement is done
#			on the raw image pair. This is supposed to 
#			reduce the effect of a difference in gain and
#			contrast between the L/R cameras and improve the
#			correlation matching rate. (on = 1; off = 0).
#
# DO_EMBOSS		convolue the raw input images with the Matrix M
#				    [-1 -1 0]
#			where	M = [-1  0 1]
#				    [ 0  1 1]
#			This perform a type of emboss which suppress
#			the low frequency signal in the images. This
#			improve greatly the matching rates for image
#			pair having different gains in between the
#			left and right images (on = 1; off = 0).
#
# DO_SECOND_HIST_EQ	same as the DO_FIRST_HIST_EQ but it is done on
#			the embossed images (on = 1; off = 0).
#
#
# ALIGN.*		linear transformation matrix to be applied to
#			the left image, note that the DO_VERT_CAL uses
#			it. The new gray intensity pixel value is
#			computed by bilinear approximation.
#
#				| a b Tx |	     | col |
#			Align = | c d Ty |   Pixel = | row |
#				| 0 0 1  |	     |  1  |
#
# DO_VERT_CAL		when enabled (1) it performs a alignment of
#			the left image relatively to the right
#			one. The alignment is done in rotation, scale
#			and vertical offset. the center pixel of the
#			left image is not moved horizontally so the
#			toe in still valid. Note: reset the ALIGN
#			matrix if DO_VERT_CAL is used because
#			otherwise two reprojection on the left image
#			will be done and that will degrade the quality
#			of the models. The routine uses a least square
#			best fit on a user defined number of tie points.
#
#
# H_TIE_PTS &		number of tie points used by the DO_VERT_CAL
# V_TIE_PTS		procedure. the tie points will be homogenously
#			distributed over the image.
#
# MIN_V_SHIFT &		upper and lower image shift limits for the 
# MAX_V_SHIFT		automatic image alignment calibration. [pixels]
#
# WRITE_PREPROCESSED	write the preprocessed images to the disk as a
#			.pgm file (on = 1; off = 0).
#
# WRITE_TEXTURE		write the texture file to the disk as a
#			.pgm file (on = 1; off = 0). This file will
#			need to be converted to an appropriate file
#			format readable by the renderer (.rgb on SGIs)
#			(on = 1; off = 0).
#
# APPLY_MASK		when active (1) this variable launch a routine
#			that automatically identify the region of the 
#			image that will have a poor matching rate and 
#			create a mask that will be used to remove the 
#			disparity matches under these regions. The mask
#			is saved with the disparity map in stp format.
#
#
# WRITE_MASK		Write the mask to a pgm file with the texture
#			on top and the mask on the bottom. note that
#			this file is not reloadable into the pipeline
#			but the mask is also saved with the disparity map.
#
# MASK_LOW_CONTRAST_TRESHOLD set the treshold value for the pixel to
#			be masked. It uses the a metric measuring the
#			amount of texture under the larger kernel
#			(1st, 2nd, 3rd pass). if the average variation
#			in gray level, for each pixel compared to the
#			average gray level under the kernel, over the
#			image is smaller the this treshold the pixel
#			is masked. see code for details.
#
# CROP_*		These are the image pixel locations for the
#			top left corner and bottom right corner of
#			coordiates for the cropping window.  This is
#			activated when the DO_FORMAT_IMG_SIZE is on.
#
# OUT_WIDTH OUT_HEIGHT	Size of the subSampled copy of the raw stereo
#			image.  This is only done if DO_FORMAT_IMG_SIZE is on.
#			If one of the two is equal to 0 the pipeline
#			will conserve the original aspect ratio.


# DO_CORRELATION	when active (1) turn on the correlation.
#

# CORR_1_CLEAN_UP	when active (1) perform a clean up on the disp
#			map created by the 1st pass
#
# DO_2nd_PASS		when active (1) do the second pass using the
#			disparity range which was determined by the
#			first pass as a bound for the disparity search
#			window. This pass also compute the disparity
#			at a subpixel level.
#
# CORR_2_CLEAN_UP	when active (1) perform a clean up on the disp
#			map created by the 2nd pass
#
# DO_3rd_PASS		when active (1) perform the third pass using
#			the disparity map which was computed during
#			the second correlation to bound the disparity
#			search window for each pixel and correct for
#			the paralax artifact of the kernel.
#
# WRITE_DEBUG_DISP	when active (1) write intermediate disparity
#			map as pgm files.  These files will be named
#			[NAME]D1.pgm [NAME]D2.pgm [NAME]F1.pgm [NAME]F2.pgm
#			D is the raw disparity map and F is the
#			disparity map which has been cleaned by
#			ADP_HEAVY_CLEAN_UP and ADJ_HEAVY_CLEAN_UP.
#
# WRITE_DISP_PGM	when active (1) write the final raw disparity map
#			in a pgm format.  The STP format is
#			automatically written.  The pgm format is used
#			only for supervisory control.  Raw disparity
#			map has the extension [NAME]D.pgm or [NAME]D.stp
#
# SUB_PXL_TRESHOLD	(any positive number) if the difference of the
#			sub pixel disparity calculated by the
#			correlation and the cross correlation  is
#			greater than this value reject the match.  
#
# H_KERNEL_1 &		(any pos odd number) a typical value is (11,11)
# V_KERNE		this is the size in pixels of the kernel used
#			for the first correlation pass.
#
# CORR_MARGIN_2		(any pos number) a typical value is 3-5
#			the search window for the second pass is
#			extended by +/- this value.	
#
# H_KERNEL_2		(any pos odd number)a typical value is (11,11)
# V_KERNEL_2		this is the size in pixels of the kernel used
#			for the second correlation pass.
#
# CORR_MARGIN_3		(any pos number) a typical value is 3-5 the 
#			search window for each pixel for the third pass is
#			extended by +/- this value.
#
# H_KERNEL_3		((7,7)->(21,21)) this is the size in pixels of the 
# V_KERNEL_3		kernel used for the third correlation pass.
#
# H_CORR_MAX		(stereo head dependant value) search window
# H_CORR_MIN		for the first pass
#

# DO_DOTCLOUD			when active (1) compute range map and
#				create a dot cloud from the disparity map
#
# FAR_FIELD_BILLBOARD		when active(1) place verticies further
#				than UNIVERSE_RADIUS on a billboard.
#				If not active(0) discard verticies
#				further than UNIVERSE_RADIUS.
#
# DO_LOCAL_LEVEL_TRANSFORM	when active(1) change reference frame
#				(used for path finder only) RTFS!
#
# WRITE_DOTCLOUD		when active(1) write a dot cloud file
#				file name [NAME].dotcloud 
#				vertices missing range values are
#				places at the origin.
#
# PAN_OFFSET			(float in deg) offset for the
# TILT_OFFSET			direction the camera is pointing
# 
# UNIVERSE_RADIUS		(float distance meter) this is the radial
#				size of the terrain (pizza) from the camera.
#
# MODEL_SCALE			(float scale factor) output scale of
#				the model in meters CTFS!
#
# X_OFFSET			(float distance meter) this is the translation
# Y_OFFSET			of the camera from the origin of the world
# Z_OFFSET
#
# IMP_AZ_OFFSET, X_IMP_OFFSET, Y_IMP_OFFSET, Z_IMP_OFFSET: 
#				pathfinder stuff.
# LOCAL_LEVEL_W, LOCAL_LEVEL_X LOCAL_LEVEL_Y, LOCAL_LEVEL_Z:
#				Pathfinder coordinate transform.


# DO_3D_MESH			when active (1) create a mesh
#				out of the dotcloud.
#
# DOUBLE_SIDED			when inactive (0) remove the back
#				face in the nff file.
#
# NFF_PLAIN			when active (1), write an
#				output file in nff format
#				without texture.
#
# NFF_TXT			when active (1), write an
#				output file in nff format with
#				texture.
#
# INVENTOR			when active (1), write an
#				output file in IV format.
#
# VRML				when active (1), write an
#				output file in VRML format.
#
# NFF_2D_MAP			when active (1) force all z value at
#				0.0 and build a flat terrain
#
# NFF_V_STEP			RTFC!		
# NFF_H_STEP
#
# NFF_MAX_JUMP			max connecting distance in pixel
#				disparity value


# end of file

END
#end
